@@ -1,26 +1,23 @@
-# -*- coding: utf-8 -*-
 """
 ==========================
 Bipartite Graph Algorithms
 ==========================
 """
-#    Copyright (C) 2013-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
 import networkx as nx
-__author__ = """\n""".join(['Jordi Torrents <jtorrents@milnou.net>',
-                            'Aric Hagberg <aric.hagberg@gmail.com>'])
-__all__ = ['is_bipartite',
-           'is_bipartite_node_set',
-           'color',
-           'sets',
-           'density',
-           'degrees']
+from networkx.algorithms.components import connected_components
+from networkx.exception import AmbiguousSolution
+
+__all__ = [
+    "is_bipartite",
+    "is_bipartite_node_set",
+    "color",
+    "sets",
+    "density",
+    "degrees",
+]
 
 
+@nx._dispatchable
 def color(G):
     """Returns a two-coloring of the graph.
 
@@ -33,11 +30,12 @@ def color(G):
     Returns
     -------
     color : dictionary
-       A dictionary keyed by node with a 1 or 0 as data for each node color.
+        A dictionary keyed by node with a 1 or 0 as data for each node color.
 
     Raises
     ------
-    exc:`NetworkXError` if the graph is not two-colorable.
+    NetworkXError
+        If the graph is not two-colorable.
 
     Examples
     --------
@@ -47,20 +45,20 @@ def color(G):
     >>> print(c)
     {0: 1, 1: 0, 2: 1, 3: 0}
 
-    You can use this to set a node attribute indicating the biparite set:
+    You can use this to set a node attribute indicating the bipartite set:
 
-    >>> nx.set_node_attributes(G, c, 'bipartite')
-    >>> print(G.nodes[0]['bipartite'])
+    >>> nx.set_node_attributes(G, c, "bipartite")
+    >>> print(G.nodes[0]["bipartite"])
     1
-    >>> print(G.nodes[1]['bipartite'])
+    >>> print(G.nodes[1]["bipartite"])
     0
     """
     if G.is_directed():
         import itertools
 
         def neighbors(v):
-            return itertools.chain.from_iterable([G.predecessors(v),
-                                                  G.successors(v)])
+            return itertools.chain.from_iterable([G.predecessors(v), G.successors(v)])
+
     else:
         neighbors = G.neighbors
 
@@ -85,8 +83,9 @@ def color(G):
     return color
 
 
+@nx._dispatchable
 def is_bipartite(G):
-    """ Returns True if graph G is bipartite, False if not.
+    """Returns True if graph G is bipartite, False if not.
 
     Parameters
     ----------
@@ -110,6 +109,7 @@ def is_bipartite(G):
         return False
 
 
+@nx._dispatchable
 def is_bipartite_node_set(G, nodes):
     """Returns True if nodes and G/nodes are a bipartition of G.
 
@@ -124,24 +124,37 @@ def is_bipartite_node_set(G, nodes):
     --------
     >>> from networkx.algorithms import bipartite
     >>> G = nx.path_graph(4)
-    >>> X = set([1,3])
-    >>> bipartite.is_bipartite_node_set(G,X)
+    >>> X = set([1, 3])
+    >>> bipartite.is_bipartite_node_set(G, X)
     True
 
     Notes
     -----
+    An exception is raised if the input nodes are not distinct, because in this
+    case some bipartite algorithms will yield incorrect results.
     For connected graphs the bipartite sets are unique.  This function handles
     disconnected graphs.
     """
     S = set(nodes)
-    for CC in nx.connected_component_subgraphs(G):
+
+    if len(S) < len(nodes):
+        # this should maybe just return False?
+        raise AmbiguousSolution(
+            "The input node set contains duplicates.\n"
+            "This may lead to incorrect results when using it in bipartite algorithms.\n"
+            "Consider using set(nodes) as the input"
+        )
+
+    for CC in (G.subgraph(c).copy() for c in connected_components(G)):
         X, Y = sets(CC)
-        if not ((X.issubset(S) and Y.isdisjoint(S)) or
-                (Y.issubset(S) and X.isdisjoint(S))):
+        if not (
+            (X.issubset(S) and Y.isdisjoint(S)) or (Y.issubset(S) and X.isdisjoint(S))
+        ):
             return False
     return True
 
 
+@nx._dispatchable
 def sets(G, top_nodes=None):
     """Returns bipartite node sets of graph G.
 
@@ -154,28 +167,26 @@ def sets(G, top_nodes=None):
     ----------
     G : NetworkX graph
 
-    top_nodes : container
-
+    top_nodes : container, optional
       Container with all nodes in one bipartite node set. If not supplied
       it will be computed. But if more than one solution exists an exception
       will be raised.
 
     Returns
     -------
-    (X,Y) : two-tuple of sets
-       One set of nodes for each part of the bipartite graph.
+    X : set
+      Nodes from one side of the bipartite graph.
+    Y : set
+      Nodes from the other side.
 
     Raises
     ------
-    AmbiguousSolution : Exception
-
+    AmbiguousSolution
       Raised if the input bipartite graph is disconnected and no container
       with all nodes in one bipartite set is provided. When determining
       the nodes in each bipartite set more than one valid solution is
       possible if the input graph is disconnected.
-
-    NetworkXError: Exception
-
+    NetworkXError
       Raised if the input graph is not bipartite.
 
     Examples
@@ -202,7 +213,7 @@ def sets(G, top_nodes=None):
         Y = set(G) - X
     else:
         if not is_connected(G):
-            msg = 'Disconnected graph: Ambiguous solution for bipartite sets.'
+            msg = "Disconnected graph: Ambiguous solution for bipartite sets."
             raise nx.AmbiguousSolution(msg)
         c = color(G)
         X = {n for n, is_top in c.items() if is_top}
@@ -210,12 +221,13 @@ def sets(G, top_nodes=None):
     return (X, Y)
 
 
+@nx._dispatchable(graphs="B")
 def density(B, nodes):
     """Returns density of bipartite graph B.
 
     Parameters
     ----------
-    G : NetworkX graph
+    B : NetworkX graph
 
     nodes: list or container
       Nodes in one node set of the bipartite graph.
@@ -228,12 +240,12 @@ def density(B, nodes):
     Examples
     --------
     >>> from networkx.algorithms import bipartite
-    >>> G = nx.complete_bipartite_graph(3,2)
-    >>> X=set([0,1,2])
-    >>> bipartite.density(G,X)
+    >>> G = nx.complete_bipartite_graph(3, 2)
+    >>> X = set([0, 1, 2])
+    >>> bipartite.density(G, X)
     1.0
-    >>> Y=set([3,4])
-    >>> bipartite.density(G,Y)
+    >>> Y = set([3, 4])
+    >>> bipartite.density(G, Y)
     1.0
 
     Notes
@@ -256,18 +268,19 @@ def density(B, nodes):
         d = 0.0
     else:
         if B.is_directed():
-            d = m / (2.0 * float(nb * nt))
+            d = m / (2 * nb * nt)
         else:
-            d = m / float(nb * nt)
+            d = m / (nb * nt)
     return d
 
 
+@nx._dispatchable(graphs="B", edge_attrs="weight")
 def degrees(B, nodes, weight=None):
     """Returns the degrees of the two node sets in the bipartite graph B.
 
     Parameters
     ----------
-    G : NetworkX graph
+    B : NetworkX graph
 
     nodes: list or container
       Nodes in one node set of the bipartite graph.
@@ -285,9 +298,9 @@ def degrees(B, nodes, weight=None):
     Examples
     --------
     >>> from networkx.algorithms import bipartite
-    >>> G = nx.complete_bipartite_graph(3,2)
-    >>> Y=set([3,4])
-    >>> degX,degY=bipartite.degrees(G,Y)
+    >>> G = nx.complete_bipartite_graph(3, 2)
+    >>> Y = set([3, 4])
+    >>> degX, degY = bipartite.degrees(G, Y)
     >>> dict(degX)
     {0: 2, 1: 2, 2: 2}
 
