@@ -1,31 +1,23 @@
-# -*- coding: utf-8 -*-
-#
-#    Copyright (C) 2004-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
 """Algorithms to characterize the number of triangles in a graph."""
-from __future__ import division
 
-from itertools import chain
-from itertools import combinations
 from collections import Counter
+from itertools import chain, combinations
 
 import networkx as nx
 from networkx.utils import not_implemented_for
 
-__author__ = """\n""".join(['Aric Hagberg <aric.hagberg@gmail.com>',
-                            'Dan Schult (dschult@colgate.edu)',
-                            'Pieter Swart (swart@lanl.gov)',
-                            'Jordi Torrents <jtorrents@milnou.net>'])
-
-__all__ = ['triangles', 'average_clustering', 'clustering', 'transitivity',
-           'square_clustering', 'generalized_degree']
+__all__ = [
+    "triangles",
+    "average_clustering",
+    "clustering",
+    "transitivity",
+    "square_clustering",
+    "generalized_degree",
+]
 
 
-@not_implemented_for('directed')
+@not_implemented_for("directed")
+@nx._dispatchable
 def triangles(G, nodes=None):
     """Compute the number of triangles.
 
@@ -35,42 +27,69 @@ def triangles(G, nodes=None):
     ----------
     G : graph
        A networkx graph
-    nodes : container of nodes, optional (default= all nodes in G)
-       Compute triangles for nodes in this container.
+
+    nodes : node, iterable of nodes, or None (default=None)
+        If a singleton node, return the number of triangles for that node.
+        If an iterable, compute the number of triangles for each of those nodes.
+        If `None` (the default) compute the number of triangles for all nodes in `G`.
 
     Returns
     -------
-    out : dictionary
-       Number of triangles keyed by node label.
+    out : dict or int
+       If `nodes` is a container of nodes, returns number of triangles keyed by node (dict).
+       If `nodes` is a specific node, returns number of triangles for the node (int).
 
     Examples
     --------
-    >>> G=nx.complete_graph(5)
-    >>> print(nx.triangles(G,0))
+    >>> G = nx.complete_graph(5)
+    >>> print(nx.triangles(G, 0))
     6
     >>> print(nx.triangles(G))
     {0: 6, 1: 6, 2: 6, 3: 6, 4: 6}
-    >>> print(list(nx.triangles(G,(0,1)).values()))
+    >>> print(list(nx.triangles(G, [0, 1]).values()))
     [6, 6]
 
     Notes
     -----
-    When computing triangles for the entire graph each triangle is counted
-    three times, once at each node.  Self loops are ignored.
+    Self loops are ignored.
 
     """
-    # If `nodes` represents a single node in the graph, return only its number
-    # of triangles.
-    if nodes in G:
-        return next(_triangles_and_degree_iter(G, nodes))[2] // 2
-    # Otherwise, `nodes` represents an iterable of nodes, so return a
-    # dictionary mapping node to number of triangles.
-    return {v: t // 2 for v, d, t, _ in _triangles_and_degree_iter(G, nodes)}
+    if nodes is not None:
+        # If `nodes` represents a single node, return only its number of triangles
+        if nodes in G:
+            return next(_triangles_and_degree_iter(G, nodes))[2] // 2
+
+        # if `nodes` is a container of nodes, then return a
+        # dictionary mapping node to number of triangles.
+        return {v: t // 2 for v, d, t, _ in _triangles_and_degree_iter(G, nodes)}
+
+    # if nodes is None, then compute triangles for the complete graph
+
+    # dict used to avoid visiting the same nodes twice
+    # this allows calculating/counting each triangle only once
+    later_nbrs = {}
+
+    # iterate over the nodes in a graph
+    for node, neighbors in G.adjacency():
+        later_nbrs[node] = {n for n in neighbors if n not in later_nbrs and n != node}
+
+    # instantiate Counter for each node to include isolated nodes
+    # add 1 to the count if a nodes neighbor's neighbor is also a neighbor
+    triangle_counts = Counter(dict.fromkeys(G, 0))
+    for node1, neighbors in later_nbrs.items():
+        for node2 in neighbors:
+            third_nodes = neighbors & later_nbrs[node2]
+            m = len(third_nodes)
+            triangle_counts[node1] += m
+            triangle_counts[node2] += m
+            triangle_counts.update(third_nodes)
+
+    return dict(triangle_counts)
 
 
-@not_implemented_for('multigraph')
+@not_implemented_for("multigraph")
 def _triangles_and_degree_iter(G, nodes=None):
-    """ Return an iterator of (node, degree, triangles, generalized degree).
+    """Return an iterator of (node, degree, triangles, generalized degree).
 
     This double counts triangles so you may want to divide by 2.
     See degree(), triangles() and generalized_degree() for definitions
@@ -89,13 +108,18 @@ def _triangles_and_degree_iter(G, nodes=None):
         yield (v, len(vs), ntriangles, gen_degree)
 
 
-@not_implemented_for('multigraph')
-def _weighted_triangles_and_degree_iter(G, nodes=None, weight='weight'):
-    """ Return an iterator of (node, degree, weighted_triangles).
+@not_implemented_for("multigraph")
+def _weighted_triangles_and_degree_iter(G, nodes=None, weight="weight"):
+    """Return an iterator of (node, degree, weighted_triangles).
 
     Used for weighted clustering.
+    Note: this returns the geometric average weight of edges in the triangle.
+    Also, each triangle is counted twice (each direction).
+    So you may want to divide by 2.
 
     """
+    import numpy as np
+
     if weight is None or G.number_of_edges() == 0:
         max_weight = 1
     else:
@@ -114,22 +138,25 @@ def _weighted_triangles_and_degree_iter(G, nodes=None, weight='weight'):
         seen = set()
         for j in inbrs:
             seen.add(j)
-            # This prevents double counting.
+            # This avoids counting twice -- we double at the end.
             jnbrs = set(G[j]) - seen
             # Only compute the edge weight once, before the inner inner
             # loop.
             wij = wt(i, j)
-            weighted_triangles += sum((wij * wt(j, k) * wt(k, i)) ** (1 / 3)
-                                      for k in inbrs & jnbrs)
-        yield (i, len(inbrs), 2 * weighted_triangles)
+            weighted_triangles += np.cbrt(
+                [(wij * wt(j, k) * wt(k, i)) for k in inbrs & jnbrs]
+            ).sum()
+        yield (i, len(inbrs), 2 * float(weighted_triangles))
 
 
-@not_implemented_for('multigraph')
+@not_implemented_for("multigraph")
 def _directed_triangles_and_degree_iter(G, nodes=None):
-    """ Return an iterator of
+    """Return an iterator of
     (node, total_degree, reciprocal_degree, directed_triangles).
 
     Used for directed clustering.
+    Note that unlike `_triangles_and_degree_iter()`, this function counts
+    directed triangles so does not count triangles twice.
 
     """
     nodes_nbrs = ((n, G._pred[n], G._succ[n]) for n in G.nbunch_iter(nodes))
@@ -142,24 +169,32 @@ def _directed_triangles_and_degree_iter(G, nodes=None):
         for j in chain(ipreds, isuccs):
             jpreds = set(G._pred[j]) - {j}
             jsuccs = set(G._succ[j]) - {j}
-            directed_triangles += sum((1 for k in
-                                       chain((ipreds & jpreds),
-                                             (ipreds & jsuccs),
-                                             (isuccs & jpreds),
-                                             (isuccs & jsuccs))))
+            directed_triangles += sum(
+                1
+                for k in chain(
+                    (ipreds & jpreds),
+                    (ipreds & jsuccs),
+                    (isuccs & jpreds),
+                    (isuccs & jsuccs),
+                )
+            )
         dtotal = len(ipreds) + len(isuccs)
         dbidirectional = len(ipreds & isuccs)
         yield (i, dtotal, dbidirectional, directed_triangles)
 
 
-@not_implemented_for('multigraph')
-def _directed_weighted_triangles_and_degree_iter(G, nodes=None, weight = 'weight'):
-    """ Return an iterator of
+@not_implemented_for("multigraph")
+def _directed_weighted_triangles_and_degree_iter(G, nodes=None, weight="weight"):
+    """Return an iterator of
     (node, total_degree, reciprocal_degree, directed_weighted_triangles).
 
     Used for directed weighted clustering.
+    Note that unlike `_weighted_triangles_and_degree_iter()`, this function counts
+    directed triangles so does not count triangles twice.
 
     """
+    import numpy as np
+
     if weight is None or G.number_of_edges() == 0:
         max_weight = 1
     else:
@@ -178,32 +213,41 @@ def _directed_weighted_triangles_and_degree_iter(G, nodes=None, weight = 'weight
         for j in ipreds:
             jpreds = set(G._pred[j]) - {j}
             jsuccs = set(G._succ[j]) - {j}
-            directed_triangles += sum((wt(j, i) * wt(k, i) * wt(k, j))**(1 / 3)
-                                      for k in ipreds & jpreds)
-            directed_triangles += sum((wt(j, i) * wt(k, i) * wt(j, k))**(1 / 3)
-                                      for k in ipreds & jsuccs)
-            directed_triangles += sum((wt(j, i) * wt(i, k) * wt(k, j))**(1 / 3)
-                                      for k in isuccs & jpreds)
-            directed_triangles += sum((wt(j, i) * wt(i, k) * wt(j, k))**(1 / 3)
-                                      for k in isuccs & jsuccs)
+            directed_triangles += np.cbrt(
+                [(wt(j, i) * wt(k, i) * wt(k, j)) for k in ipreds & jpreds]
+            ).sum()
+            directed_triangles += np.cbrt(
+                [(wt(j, i) * wt(k, i) * wt(j, k)) for k in ipreds & jsuccs]
+            ).sum()
+            directed_triangles += np.cbrt(
+                [(wt(j, i) * wt(i, k) * wt(k, j)) for k in isuccs & jpreds]
+            ).sum()
+            directed_triangles += np.cbrt(
+                [(wt(j, i) * wt(i, k) * wt(j, k)) for k in isuccs & jsuccs]
+            ).sum()
 
         for j in isuccs:
             jpreds = set(G._pred[j]) - {j}
             jsuccs = set(G._succ[j]) - {j}
-            directed_triangles += sum((wt(i, j) * wt(k, i) * wt(k, j))**(1 / 3)
-                                      for k in ipreds & jpreds)
-            directed_triangles += sum((wt(i, j) * wt(k, i) * wt(j, k))**(1 / 3)
-                                      for k in ipreds & jsuccs)
-            directed_triangles += sum((wt(i, j) * wt(i, k) * wt(k, j))**(1 / 3)
-                                      for k in isuccs & jpreds)
-            directed_triangles += sum((wt(i, j) * wt(i, k) * wt(j, k))**(1 / 3)
-                                      for k in isuccs & jsuccs)
+            directed_triangles += np.cbrt(
+                [(wt(i, j) * wt(k, i) * wt(k, j)) for k in ipreds & jpreds]
+            ).sum()
+            directed_triangles += np.cbrt(
+                [(wt(i, j) * wt(k, i) * wt(j, k)) for k in ipreds & jsuccs]
+            ).sum()
+            directed_triangles += np.cbrt(
+                [(wt(i, j) * wt(i, k) * wt(k, j)) for k in isuccs & jpreds]
+            ).sum()
+            directed_triangles += np.cbrt(
+                [(wt(i, j) * wt(i, k) * wt(j, k)) for k in isuccs & jsuccs]
+            ).sum()
 
         dtotal = len(ipreds) + len(isuccs)
         dbidirectional = len(ipreds & isuccs)
-        yield (i, dtotal, dbidirectional, directed_triangles)
+        yield (i, dtotal, dbidirectional, float(directed_triangles))
 
 
+@nx._dispatchable(edge_attrs="weight")
 def average_clustering(G, nodes=None, weight=None, count_zeros=True):
     r"""Compute the average clustering coefficient for the graph G.
 
@@ -236,7 +280,7 @@ def average_clustering(G, nodes=None, weight=None, count_zeros=True):
 
     Examples
     --------
-    >>> G=nx.complete_graph(5)
+    >>> G = nx.complete_graph(5)
     >>> print(nx.average_clustering(G))
     1.0
 
@@ -259,10 +303,11 @@ def average_clustering(G, nodes=None, weight=None, count_zeros=True):
     """
     c = clustering(G, nodes, weight=weight).values()
     if not count_zeros:
-        c = [v for v in c if v > 0]
+        c = [v for v in c if abs(v) > 0]
     return sum(c) / len(c)
 
 
+@nx._dispatchable(edge_attrs="weight")
 def clustering(G, nodes=None, weight=None):
     r"""Compute the clustering coefficient for nodes.
 
@@ -290,26 +335,30 @@ def clustering(G, nodes=None, weight=None):
 
     The value of :math:`c_u` is assigned to 0 if :math:`deg(u) < 2`.
 
+    Additionally, this weighted definition has been generalized to support negative edge weights [3]_.
+
     For directed graphs, the clustering is similarly defined as the fraction
     of all possible directed triangles or geometric average of the subgraph
-    edge weights for unweighted and weighted directed graph respectively [3]_.
+    edge weights for unweighted and weighted directed graph respectively [4]_.
 
     .. math::
 
-       c_u = \frac{1}{deg^{tot}(u)(deg^{tot}(u)-1) - 2deg^{\leftrightarrow}(u)}
-             T(u),
+       c_u = \frac{T(u)}{2(deg^{tot}(u)(deg^{tot}(u)-1) - 2deg^{\leftrightarrow}(u))},
 
     where :math:`T(u)` is the number of directed triangles through node
     :math:`u`, :math:`deg^{tot}(u)` is the sum of in degree and out degree of
     :math:`u` and :math:`deg^{\leftrightarrow}(u)` is the reciprocal degree of
     :math:`u`.
 
+
     Parameters
     ----------
     G : graph
 
-    nodes : container of nodes, optional (default=all nodes in G)
-       Compute clustering for nodes in this container.
+    nodes : node, iterable of nodes, or None (default=None)
+        If a singleton node, return the number of triangles for that node.
+        If an iterable, compute the number of triangles for each of those nodes.
+        If `None` (the default) compute the number of triangles for all nodes in `G`.
 
     weight : string or None, optional (default=None)
        The edge attribute that holds the numerical value used as a weight.
@@ -322,8 +371,8 @@ def clustering(G, nodes=None, weight=None):
 
     Examples
     --------
-    >>> G=nx.complete_graph(5)
-    >>> print(nx.clustering(G,0))
+    >>> G = nx.complete_graph(5)
+    >>> print(nx.clustering(G, 0))
     1.0
     >>> print(nx.clustering(G))
     {0: 1.0, 1: 1.0, 2: 1.0, 3: 1.0, 4: 1.0}
@@ -341,34 +390,39 @@ def clustering(G, nodes=None, weight=None):
     .. [2] Intensity and coherence of motifs in weighted complex
        networks by J. P. Onnela, J. Saramäki, J. Kertész, and K. Kaski,
        Physical Review E, 71(6), 065103 (2005).
-    .. [3] Clustering in complex directed networks by G. Fagiolo,
+    .. [3] Generalization of Clustering Coefficients to Signed Correlation Networks
+       by G. Costantini and M. Perugini, PloS one, 9(2), e88669 (2014).
+    .. [4] Clustering in complex directed networks by G. Fagiolo,
        Physical Review E, 76(2), 026107 (2007).
     """
     if G.is_directed():
         if weight is not None:
-            td_iter = _directed_weighted_triangles_and_degree_iter(
-                G, nodes, weight)
-            clusterc = {v: 0 if t == 0 else t / ((dt * (dt - 1) - 2 * db) * 2)
-                        for v, dt, db, t in td_iter}
+            td_iter = _directed_weighted_triangles_and_degree_iter(G, nodes, weight)
+            clusterc = {
+                v: 0 if t == 0 else t / ((dt * (dt - 1) - 2 * db) * 2)
+                for v, dt, db, t in td_iter
+            }
         else:
             td_iter = _directed_triangles_and_degree_iter(G, nodes)
-            clusterc = {v: 0 if t == 0 else t / ((dt * (dt - 1) - 2 * db) * 2)
-                        for v, dt, db, t in td_iter}
+            clusterc = {
+                v: 0 if t == 0 else t / ((dt * (dt - 1) - 2 * db) * 2)
+                for v, dt, db, t in td_iter
+            }
     else:
+        # The formula 2*T/(d*(d-1)) from docs is t/(d*(d-1)) here b/c t==2*T
         if weight is not None:
             td_iter = _weighted_triangles_and_degree_iter(G, nodes, weight)
-            clusterc = {v: 0 if t == 0 else t / (d * (d - 1)) for
-                        v, d, t in td_iter}
+            clusterc = {v: 0 if t == 0 else t / (d * (d - 1)) for v, d, t in td_iter}
         else:
             td_iter = _triangles_and_degree_iter(G, nodes)
-            clusterc = {v: 0 if t == 0 else t / (d * (d - 1)) for
-                        v, d, t, _ in td_iter}
+            clusterc = {v: 0 if t == 0 else t / (d * (d - 1)) for v, d, t, _ in td_iter}
     if nodes in G:
         # Return the value of the sole entry in the dictionary.
         return clusterc[nodes]
     return clusterc
 
 
+@nx._dispatchable
 def transitivity(G):
     r"""Compute graph transitivity, the fraction of all possible triangles
     present in G.
@@ -391,19 +445,29 @@ def transitivity(G):
     out : float
        Transitivity
 
+    Notes
+    -----
+    Self loops are ignored.
+
     Examples
     --------
     >>> G = nx.complete_graph(5)
     >>> print(nx.transitivity(G))
     1.0
     """
-    triangles = sum(t for v, d, t, _ in _triangles_and_degree_iter(G))
-    contri = sum(d * (d - 1) for v, d, t, _ in _triangles_and_degree_iter(G))
+    triangles_contri = [
+        (t, d * (d - 1)) for v, d, t, _ in _triangles_and_degree_iter(G)
+    ]
+    # If the graph is empty
+    if len(triangles_contri) == 0:
+        return 0
+    triangles, contri = map(sum, zip(*triangles_contri))
     return 0 if triangles == 0 else triangles / contri
 
 
+@nx._dispatchable
 def square_clustering(G, nodes=None):
-    r""" Compute the squares clustering coefficient for nodes.
+    r"""Compute the squares clustering coefficient for nodes.
 
     For each node return the fraction of possible squares that exist at
     the node [1]_
@@ -415,9 +479,9 @@ def square_clustering(G, nodes=None):
 
     where :math:`q_v(u,w)` are the number of common neighbors of :math:`u` and
     :math:`w` other than :math:`v` (ie squares), and :math:`a_v(u,w) = (k_u -
-    (1+q_v(u,w)+\theta_{uv}))(k_w - (1+q_v(u,w)+\theta_{uw}))`, where
+    (1+q_v(u,w)+\theta_{uv})) + (k_w - (1+q_v(u,w)+\theta_{uw}))`, where
     :math:`\theta_{uw} = 1` if :math:`u` and :math:`w` are connected and 0
-    otherwise.
+    otherwise. [2]_
 
     Parameters
     ----------
@@ -433,8 +497,8 @@ def square_clustering(G, nodes=None):
 
     Examples
     --------
-    >>> G=nx.complete_graph(5)
-    >>> print(nx.square_clustering(G,0))
+    >>> G = nx.complete_graph(5)
+    >>> print(nx.square_clustering(G, 0))
     1.0
     >>> print(nx.square_clustering(G))
     {0: 1.0, 1: 1.0, 2: 1.0, 3: 1.0, 4: 1.0}
@@ -452,6 +516,9 @@ def square_clustering(G, nodes=None):
     .. [1] Pedro G. Lind, Marta C. González, and Hans J. Herrmann. 2005
         Cycles and clustering in bipartite networks.
         Physical Review E (72) 056127.
+    .. [2] Zhang, Peng et al. Clustering Coefficient and Community Structure of
+        Bipartite Networks. Physica A: Statistical Mechanics and its Applications 387.27 (2008): 6869–6875.
+        https://arxiv.org/abs/0710.0117v1
     """
     if nodes is None:
         node_iter = G
@@ -462,12 +529,12 @@ def square_clustering(G, nodes=None):
         clustering[v] = 0
         potential = 0
         for u, w in combinations(G[v], 2):
-            squares = len((set(G[u]) & set(G[w])) - set([v]))
+            squares = len((set(G[u]) & set(G[w])) - {v})
             clustering[v] += squares
             degm = squares + 1
             if w in G[u]:
                 degm += 1
-            potential += (len(G[u]) - degm) * (len(G[w]) - degm) + squares
+            potential += (len(G[u]) - degm) + (len(G[w]) - degm) + squares
         if potential > 0:
             clustering[v] /= potential
     if nodes in G:
@@ -476,9 +543,10 @@ def square_clustering(G, nodes=None):
     return clustering
 
 
-@not_implemented_for('directed')
+@not_implemented_for("directed")
+@nx._dispatchable
 def generalized_degree(G, nodes=None):
-    r""" Compute the generalized degree for nodes.
+    r"""Compute the generalized degree for nodes.
 
     For each node, the generalized degree shows how many edges of given
     triangle multiplicity the node is connected to. The triangle multiplicity
@@ -503,21 +571,23 @@ def generalized_degree(G, nodes=None):
 
     Examples
     --------
-    >>> G=nx.complete_graph(5)
-    >>> print(nx.generalized_degree(G,0))
+    >>> G = nx.complete_graph(5)
+    >>> print(nx.generalized_degree(G, 0))
     Counter({3: 4})
     >>> print(nx.generalized_degree(G))
     {0: Counter({3: 4}), 1: Counter({3: 4}), 2: Counter({3: 4}), 3: Counter({3: 4}), 4: Counter({3: 4})}
 
     To recover the number of triangles attached to a node:
 
-    >>> k1 = nx.generalized_degree(G,0)
-    >>> sum([k*v for k,v in k1.items()])/2 == nx.triangles(G,0)
+    >>> k1 = nx.generalized_degree(G, 0)
+    >>> sum([k * v for k, v in k1.items()]) / 2 == nx.triangles(G, 0)
     True
 
     Notes
     -----
-    In a network of N nodes, the highest triangle multiplicty an edge can have
+    Self loops are ignored.
+
+    In a network of N nodes, the highest triangle multiplicity an edge can have
     is N-2.
 
     The return value does not include a `zero` entry if no edges of a
