@@ -24,6 +24,7 @@ __all__ = ["k_edge_augmentation", "is_k_edge_connected", "is_locally_k_edge_conn
 
 @not_implemented_for("directed")
 @not_implemented_for("multigraph")
+@nx._dispatchable
 def is_k_edge_connected(G, k):
     """Tests to see if a graph is k-edge-connected.
 
@@ -67,13 +68,14 @@ def is_k_edge_connected(G, k):
         if k == 1:
             return nx.is_connected(G)
         elif k == 2:
-            return not nx.has_bridges(G)
+            return nx.is_connected(G) and not nx.has_bridges(G)
         else:
             return nx.edge_connectivity(G, cutoff=k) >= k
 
 
 @not_implemented_for("directed")
 @not_implemented_for("multigraph")
+@nx._dispatchable
 def is_locally_k_edge_connected(G, s, t, k):
     """Tests to see if an edge in a graph is locally k-edge-connected.
 
@@ -131,6 +133,7 @@ def is_locally_k_edge_connected(G, s, t, k):
 
 @not_implemented_for("directed")
 @not_implemented_for("multigraph")
+@nx._dispatchable
 def k_edge_augmentation(G, k, avail=None, weight=None, partial=False):
     """Finds set of edges to k-edge-connect G.
 
@@ -281,6 +284,7 @@ def k_edge_augmentation(G, k, avail=None, weight=None, partial=False):
             raise
 
 
+@nx._dispatchable
 def partial_k_edge_augmentation(G, k, avail, weight=None):
     """Finds augmentation that k-edge-connects as much of the graph as possible.
 
@@ -288,7 +292,7 @@ def partial_k_edge_augmentation(G, k, avail, weight=None):
     small set of edges that partially k-edge-connects as much of the graph as
     possible. All possible edges are generated between remaining parts.
     This minimizes the number of k-edge-connected subgraphs in the resulting
-    graph and maxmizes the edge connectivity between those subgraphs.
+    graph and maximizes the edge connectivity between those subgraphs.
 
     Parameters
     ----------
@@ -383,6 +387,7 @@ def partial_k_edge_augmentation(G, k, avail, weight=None):
 
 @not_implemented_for("multigraph")
 @not_implemented_for("directed")
+@nx._dispatchable
 def one_edge_augmentation(G, avail=None, weight=None, partial=False):
     """Finds minimum weight set of edges to connect G.
 
@@ -437,6 +442,7 @@ def one_edge_augmentation(G, avail=None, weight=None, partial=False):
 
 @not_implemented_for("multigraph")
 @not_implemented_for("directed")
+@nx._dispatchable
 def bridge_augmentation(G, avail=None, weight=None):
     """Finds the a set of edges that bridge connects G.
 
@@ -572,6 +578,7 @@ def _lightest_meta_edges(mapping, avail_uv, avail_w):
             yield MetaEdge((mu, mv), (u, v), w)
 
 
+@nx._dispatchable
 def unconstrained_one_edge_augmentation(G):
     """Finds the smallest set of edges to connect G.
 
@@ -614,6 +621,7 @@ def unconstrained_one_edge_augmentation(G):
         yield (inverse[mu][0], inverse[mv][0])
 
 
+@nx._dispatchable
 def weighted_one_edge_augmentation(G, avail, weight=None, partial=False):
     """Finds the minimum weight set of edges to connect G if one exists.
 
@@ -682,6 +690,7 @@ def weighted_one_edge_augmentation(G, avail, weight=None, partial=False):
             yield edge
 
 
+@nx._dispatchable
 def unconstrained_bridge_augmentation(G):
     """Finds an optimal 2-edge-augmentation of G using the fewest edges.
 
@@ -728,7 +737,7 @@ def unconstrained_bridge_augmentation(G):
     implementation implicitly constructs T'.
 
     The leafs of T are the nodes with no existing edges in T'.
-    Order the leafs of T' by DFS prorder. Then break this list in half
+    Order the leafs of T' by DFS preorder. Then break this list in half
     and add the zipped pairs to A2.
 
     The set A = A1 + A2 is the minimum augmentation in the metagraph.
@@ -836,6 +845,7 @@ def unconstrained_bridge_augmentation(G):
                 break
 
 
+@nx._dispatchable
 def weighted_bridge_augmentation(G, avail, weight=None):
     """Finds an approximate min-weight 2-edge-augmentation of G.
 
@@ -992,7 +1002,7 @@ def weighted_bridge_augmentation(G, avail, weight=None):
 
     # For each edge e, in the branching that did not belong to the directed
     # tree T, add the corresponding edge that **GENERATED** it (this is not
-    # necesarilly e itself!)
+    # necessarily e itself!)
 
     # ensure the third case does not generate edges twice
     bridge_connectors = set()
@@ -1030,6 +1040,7 @@ def _minimum_rooted_branching(D, root):
     return A
 
 
+@nx._dispatchable(returns_graph=True)
 def collapse(G, grouped_nodes):
     """Collapses each group of nodes into a single node.
 
@@ -1057,7 +1068,7 @@ def collapse(G, grouped_nodes):
 
     Examples
     --------
-    >>> # Collapses a graph using disjoint groups, but not necesarilly connected
+    >>> # Collapses a graph using disjoint groups, but not necessarily connected
     >>> G = nx.Graph([(1, 0), (2, 3), (3, 1), (3, 4), (4, 5), (5, 6), (5, 7)])
     >>> G.add_node("A")
     >>> grouped_nodes = [{0, 1, 2, 3}, {5, 6, 7}]
@@ -1101,6 +1112,7 @@ def collapse(G, grouped_nodes):
     return C
 
 
+@nx._dispatchable
 def complement_edges(G):
     """Returns only the edges in the complement of G
 
@@ -1143,9 +1155,10 @@ def _compat_shuffle(rng, input):
     rng.shuffle(input)
 
 
-@py_random_state(4)
 @not_implemented_for("multigraph")
 @not_implemented_for("directed")
+@py_random_state(4)
+@nx._dispatchable
 def greedy_k_edge_augmentation(G, k, avail=None, weight=None, seed=None):
     """Greedy algorithm for finding a k-edge-augmentation
 
