@@ -1,11 +1,10 @@
 from io import BytesIO
-import tempfile
+
 import pytest
 
 import networkx as nx
 import networkx.readwrite.graph6 as g6
-from networkx.testing.utils import assert_edges_equal
-from networkx.testing.utils import assert_nodes_equal
+from networkx.utils import edges_equal, nodes_equal
 
 
 class TestGraph6Utils:
@@ -20,8 +19,8 @@ class TestFromGraph6Bytes:
     def test_from_graph6_bytes(self):
         data = b"DF{"
         G = nx.from_graph6_bytes(data)
-        assert_nodes_equal(G.nodes(), [0, 1, 2, 3, 4])
-        assert_edges_equal(
+        assert nodes_equal(G.nodes(), [0, 1, 2, 3, 4])
+        assert edges_equal(
             G.edges(), [(0, 3), (0, 4), (1, 3), (1, 4), (2, 3), (2, 4), (3, 4)]
         )
 
@@ -30,8 +29,8 @@ class TestFromGraph6Bytes:
         G = nx.from_graph6_bytes(data)
         fh = BytesIO(data)
         Gin = nx.read_graph6(fh)
-        assert_nodes_equal(G.nodes(), Gin.nodes())
-        assert_edges_equal(G.edges(), Gin.edges())
+        assert nodes_equal(G.nodes(), Gin.nodes())
+        assert edges_equal(G.edges(), Gin.edges())
 
 
 class TestReadGraph6:
@@ -80,9 +79,10 @@ class TestWriteGraph6:
         # The expected encoding here was verified by Sage.
         assert result.getvalue() == b"N??F~z{~Fw^_~?~?^_?\n"
 
-    def test_no_directed_graphs(self):
+    @pytest.mark.parametrize("G", (nx.MultiGraph(), nx.DiGraph()))
+    def test_no_directed_or_multi_graphs(self, G):
         with pytest.raises(nx.NetworkXNotImplemented):
-            nx.write_graph6(nx.DiGraph(), BytesIO())
+            nx.write_graph6(G, BytesIO())
 
     def test_length(self):
         for i in list(range(13)) + [31, 47, 62, 63, 64, 72]:
@@ -100,19 +100,69 @@ class TestWriteGraph6:
             nx.write_graph6(G, f)
             f.seek(0)
             H = nx.read_graph6(f)
-            assert_nodes_equal(G.nodes(), H.nodes())
-            assert_edges_equal(G.edges(), H.edges())
+            assert nodes_equal(G.nodes(), H.nodes())
+            assert edges_equal(G.edges(), H.edges())
 
-    def test_write_path(self):
-        with tempfile.NamedTemporaryFile() as f:
+    def test_write_path(self, tmp_path):
+        with open(tmp_path / "test.g6", "w+b") as f:
             g6.write_graph6_file(nx.null_graph(), f)
             f.seek(0)
             assert f.read() == b">>graph6<<?\n"
 
-    def test_relabeling(self):
-        G = nx.Graph([(0, 1)])
-        assert g6.to_graph6_bytes(G) == b">>graph6<<A_\n"
-        G = nx.Graph([(1, 2)])
-        assert g6.to_graph6_bytes(G) == b">>graph6<<A_\n"
-        G = nx.Graph([(1, 42)])
+    @pytest.mark.parametrize("edge", ((0, 1), (1, 2), (1, 42)))
+    def test_relabeling(self, edge):
+        G = nx.Graph([edge])
+        f = BytesIO()
+        nx.write_graph6(G, f)
+        f.seek(0)
+        assert f.read() == b">>graph6<<A_\n"
+
+
+class TestToGraph6Bytes:
+    def test_null_graph(self):
+        G = nx.null_graph()
+        assert g6.to_graph6_bytes(G) == b">>graph6<<?\n"
+
+    def test_trivial_graph(self):
+        G = nx.trivial_graph()
+        assert g6.to_graph6_bytes(G) == b">>graph6<<@\n"
+
+    def test_complete_graph(self):
+        assert g6.to_graph6_bytes(nx.complete_graph(4)) == b">>graph6<<C~\n"
+
+    def test_large_complete_graph(self):
+        G = nx.complete_graph(67)
+        assert g6.to_graph6_bytes(G, header=False) == b"~?@B" + b"~" * 368 + b"w\n"
+
+    def test_no_header(self):
+        G = nx.complete_graph(4)
+        assert g6.to_graph6_bytes(G, header=False) == b"C~\n"
+
+    def test_complete_bipartite_graph(self):
+        G = nx.complete_bipartite_graph(6, 9)
+        assert g6.to_graph6_bytes(G, header=False) == b"N??F~z{~Fw^_~?~?^_?\n"
+
+    @pytest.mark.parametrize("G", (nx.MultiGraph(), nx.DiGraph()))
+    def test_no_directed_or_multi_graphs(self, G):
+        with pytest.raises(nx.NetworkXNotImplemented):
+            g6.to_graph6_bytes(G)
+
+    def test_length(self):
+        for i in list(range(13)) + [31, 47, 62, 63, 64, 72]:
+            G = nx.random_graphs.gnm_random_graph(i, i * i // 4, seed=i)
+            # Strip the trailing newline.
+            gstr = g6.to_graph6_bytes(G, header=False).rstrip()
+            assert len(gstr) == ((i - 1) * i // 2 + 5) // 6 + (1 if i < 63 else 4)
+
+    def test_roundtrip(self):
+        for i in list(range(13)) + [31, 47, 62, 63, 64, 72]:
+            G = nx.random_graphs.gnm_random_graph(i, i * i // 4, seed=i)
+            data = g6.to_graph6_bytes(G)
+            H = nx.from_graph6_bytes(data.rstrip())
+            assert nodes_equal(G.nodes(), H.nodes())
+            assert edges_equal(G.edges(), H.edges())
+
+    @pytest.mark.parametrize("edge", ((0, 1), (1, 2), (1, 42)))
+    def test_relabeling(self, edge):
+        G = nx.Graph([edge])
         assert g6.to_graph6_bytes(G) == b">>graph6<<A_\n"
